Index: apps/app_parkandannounce.c
===================================================================
--- apps/app_parkandannounce.c	(revision 121364)
+++ apps/app_parkandannounce.c	(working copy)
@@ -76,14 +76,13 @@
 
 static int parkandannounce_exec(struct ast_channel *chan, void *data)
 {
-	int res=0;
 	char *return_context;
 	int lot, timeout = 0, dres;
 	char *working, *context, *exten, *priority, *dial, *dialtech, *dialstr;
 	char *template, *tpl_working, *tpl_current;
 	char *tmp[100];
 	char buf[13];
-	int looptemp=0,i=0;
+	int looptemp = 0,i = 0, res = 0;
 	char *s;
 
 	struct ast_channel *dchan;
@@ -170,10 +169,12 @@
 	/* we are using masq_park here to protect * from touching the channel once we park it.  If the channel comes out of timeout
 	before we are done announcing and the channel is messed with, Kablooeee.  So we use Masq to prevent this.  */
 
-	ast_masq_park_call(chan, NULL, timeout, &lot);
+	res = ast_masq_park_call(chan, NULL, timeout, &lot);
 
-	res=-1; 
-
+	if (lot == -1) {
+		res = -1; 
+		goto finish;
+	}
 	ast_verbose( VERBOSE_PREFIX_3 "Call Parking Called, lot: %d, timeout: %d, context: %s\n", lot, timeout, return_context);
 
 	/* Now place the call to the extention */
@@ -233,8 +234,9 @@
 	}
 
 	ast_stopstream(dchan);  
+
 	ast_hangup(dchan);
-	
+finish:
 	ast_module_user_remove(u);
 	
 	return res;
Index: res/res_features.c
===================================================================
--- res/res_features.c	(revision 121364)
+++ res/res_features.c	(working copy)
@@ -325,14 +325,30 @@
 	/* Check for channel variable PARKINGEXTEN */
 	parkingexten = pbx_builtin_getvar_helper(chan, "PARKINGEXTEN");
 	if (!ast_strlen_zero(parkingexten)) {
-		if (ast_exists_extension(NULL, parking_con, parkingexten, 1, NULL)) {
+		/*!\note The API forces us to specify a numeric parking slot, even
+		 * though the architecture would tend to support non-numeric extensions
+		 * (as are possible with SIP, for example).  Hence, we enforce that
+		 * limitation here.  If extout was not numeric, we could permit
+		 * arbitrary non-numeric extensions.
+		 */
+		if (sscanf(parkingexten, "%d", &x) != 1 || x < 0) {
+			ast_log(LOG_WARNING, "PARKINGEXTEN does not indicate a valid parking slot: '%s'.\n", parkingexten);
 			ast_mutex_unlock(&parking_lock);
 			free(pu);
+			if (extout)
+				*extout = -1;
+			return 1;	/* Continue execution if possible */
+		}
+		snprintf(pu->parkingexten, sizeof(pu->parkingexten), "%d", x);
+
+		if (ast_exists_extension(NULL, parking_con, pu->parkingexten, 1, NULL)) {
+			ast_mutex_unlock(&parking_lock);
+			free(pu);
 			ast_log(LOG_WARNING, "Requested parking extension already exists: %s@%s\n", parkingexten, parking_con);
+			if (extout)
+				*extout = -1;
 			return 1;	/* Continue execution if possible */
 		}
-		ast_copy_string(pu->parkingexten, parkingexten, sizeof(pu->parkingexten));
-		x = atoi(parkingexten);
 	} else {
 		/* Select parking space within range */
 		parking_range = parking_stop - parking_start+1;
@@ -357,6 +373,7 @@
 		/* Set pointer for next parking */
 		if (parkfindnext) 
 			parking_offset = x - parking_start + 1;
+		snprintf(pu->parkingexten, sizeof(pu->parkingexten), "%d", x);
 	}
 	
 	chan->appl = "Parked Call";
@@ -397,8 +414,6 @@
 	if (option_verbose > 1) 
 		ast_verbose(VERBOSE_PREFIX_2 "Parked %s on %d@%s. Will timeout back to extension [%s] %s, %d in %d seconds\n", pu->chan->name, pu->parkingnum, parking_con, pu->context, pu->exten, pu->priority, (pu->parkingtime/1000));
 
-	if (pu->parkingnum != -1)
-		snprintf(pu->parkingexten, sizeof(pu->parkingexten), "%d", x);
 	manager_event(EVENT_FLAG_CALL, "ParkedCall",
 		"Exten: %s\r\n"
 		"Channel: %s\r\n"
@@ -423,7 +438,7 @@
 	if (!con)	/* Still no context? Bad */
 		ast_log(LOG_ERROR, "Parking context '%s' does not exist and unable to create\n", parking_con);
 	/* Tell the peer channel the number of the parking space */
-	if (peer && ((pu->parkingnum != -1 && ast_strlen_zero(orig_chan_name)) || !strcasecmp(peer->name, orig_chan_name))) { /* Only say number if it's a number and the channel hasn't been masqueraded away */
+	if (peer && (ast_strlen_zero(orig_chan_name) || !strcasecmp(peer->name, orig_chan_name))) { /* Only say number if it's a number and the channel hasn't been masqueraded away */
 		/* Make sure we don't start saying digits to the channel being parked */
 		ast_set_flag(peer, AST_FLAG_MASQ_NOSTREAM);
 		ast_say_digits(peer, pu->parkingnum, "", peer->language);
@@ -480,6 +495,10 @@
 	orig_chan_name = ast_strdupa(chan->name);
 
 	park_call_full(chan, peer, timeout, extout, orig_chan_name);
+	if (*extout == -1) {
+		/* would be nice to play: "invalid parking extension" */
+		ast_hangup(chan);
+	}
 
 	return 0;
 }
